package org.gnurou.audioscrobbler;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.Calendar;
import java.util.TimeZone;

public class AudioscrobblerTrack extends Track implements Comparable {
	// Date the track was played, in seconds since January 1st, 1970 GMT
	private long date;

	public AudioscrobblerTrack(Track t, long date)
	{
		super(t);
		this.date = date;
	}
	
	public AudioscrobblerTrack(String title, String artist, String album, long length, long date, 
			String mbid)
	{
		super(title, artist, album, length, mbid);
		this.date = date;
	}

	public AudioscrobblerTrack(String title, String artist, long length, long date, 
			String mbid)
	{
		super(title, artist, length, mbid);
		this.date = date;
	}

	public AudioscrobblerTrack(String title, String artist, String album, long length, long date)
	{
		super(title, artist, album, length);
		this.date = date;
	}

	public AudioscrobblerTrack(String title, String artist, long length, long date)
	{
		super(title, artist, length);
		this.date = date;
	}

	public long getDate() { return date; }
	
	/**
	 * Adds the given value to the play time
	 * @param adjust
	 */
	public void adjustDate(long adjust)
	{
		date += adjust;
	}
	
	/**
	 * Returns a string of this track that can be concatenated to an Audioscrobbler track 
	 * submission URL.
	 * @throws UnsupportedEncodingException
	 */
	public String getAudioscrobblerSubmissionString() throws UnsupportedEncodingException
	{
		return (getAudioscrobblerSubmissionString(0));
	}

	/**
	 * Returns a string of this track that can be concatenated to an Audioscrobbler track 
	 * submission URL.
	 * @param nbr Index of the submitted track, for multiple-tracks requests.
	 * @throws UnsupportedEncodingException
	 */
	public String getAudioscrobblerSubmissionString(int nbr) throws UnsupportedEncodingException
	{
		String data = getLP('a', nbr) + encode(getArtist()) +
			getLP('t', nbr) + encode(getTitle()) +
			getLP('b',nbr) + encode(getAlbum()) +
			getLP('m',nbr) + encode(getMBID()) +
			getLP('l', nbr) + getLength() +
			getLP('i', nbr) + encode(formatDate());
		
		return data;
	}

	public String formatDate()
	{
		return formatDate(date);
	}

	public String formatDateToLocalTime()
	{
		return formatDate(date + TimeZone.getDefault().getOffset(date));
	}
	
	private static String formatDate(long date)
	{
		Calendar cal = Calendar.getInstance();
		cal.setTimeInMillis(date);
		
		StringWriter sw = new StringWriter();
		PrintWriter pr = new PrintWriter(sw);
		pr.printf("%04d-%02d-%02d %02d:%02d:%02d", cal.get(Calendar.YEAR),  (cal.get(Calendar.MONTH) + 1), 
				cal.get(Calendar.DAY_OF_MONTH), cal.get(Calendar.HOUR_OF_DAY), cal.get(Calendar.MINUTE), 
				cal.get(Calendar.SECOND));
		
		return sw.toString();
	}

	/**
	 * Encode the given string to UTF-8.
	 */
	private String encode(String str) throws UnsupportedEncodingException
	{
		return URLEncoder.encode(str, "UTF-8");
	}

	/**
	 * Returns the left-part of an audioscrobbler parameter.
	 * @param c The character that makes the left part.
	 * @param nbr The index of the submission.
	 */
	private String getLP(char c, int nbr)
	{
		return "&" + c + "[" + nbr + "]=";
	}
	
	public int compareTo(Object o)
	{
		if (!(o instanceof AudioscrobblerTrack)) return -1;
		AudioscrobblerTrack t = (AudioscrobblerTrack) o;
		return (int) (getDate() - t.getDate());
	}
}
