package org.gnurou.audioscrobbler;

import java.io.IOException;
import java.net.InetAddress;
import java.util.Date;
import java.util.Iterator;
import java.util.TimeZone;

import org.biffle.cliff.pearl.FileInfo;
import org.biffle.cliff.pearl.PearlException;
import org.biffle.cliff.pearl2.Pearl2TCPConnection;

/**
 * A collection that can be read from the RIO Karma.
 * @author Alexandre Courbot.
 *
 */
public class RioCollection extends Collection {
	public static final int defaultRioPort = 8302;
	
	Pearl2TCPConnection connection;
	
	public RioCollection()
	{
	}
	
	public void connect(String rioIP, String password) throws PearlException, IOException
	{
		connect(rioIP, defaultRioPort, password);
	}

	public void connect(String rioIP, int rioPort, String password) throws PearlException, IOException
	{
		connect(rioIP, rioPort);
		connection.authenticate(password);
	}

	public void closeConnection()
	{
		connection.close();
	}
	
	public void connect(String rioIP, int rioPort) throws PearlException, IOException
	{
		connection = new Pearl2TCPConnection(InetAddress.getByName(rioIP), rioPort);
	}
	
	public void buildCollection() throws PearlException
	{
		Iterator it = connection.getFileIterator();
		while (it.hasNext())
		{
			FileInfo obj = (FileInfo) it.next();
			String title = obj.getTitle();
			if (title == null) title = "";
			String artist = obj.getArtist();
			if (artist == null) artist = "";
			String album = obj.getAlbum();
			if (album == null) album = "";
			Track t = new Track(title, artist, album,
					obj.getDuration() / 1000);
			long date = obj.getLastPlayTime() * 1000;
			TrackInfo i = new TrackInfo(obj.getPlayCount(), 
					// TODO getOffset takes a GMT time, we are giving a local
					// time. It should be accurate 99% of the time still.
					new Date(date - TimeZone.getDefault().getOffset(date)).getTime());
			addTrack(t, i);
		}
	}
}
