package org.gnurou.audioscrobbler;

/**
 * This class represents a track that we can submit to an audioscrobbler server.
 * @author Alexandre Courbot.
 *
 */
public class Track {
	// Track parameters...
	private String artist;
	private String title;
	private String album;
	// Length of the track, in seconds
	private long length;
	// Musicbrainz identifier
	private String mbid;

	public Track(Track t)
	{
		this.title = t.getTitle();
		this.artist = t.getArtist();
		this.album = t.getAlbum();
		this.length = t.getLength();
		this.mbid = t.getMBID();
	}

	/**
	 * Simplified constructor, discarding the album and mbid.
	 */
	public Track(String title, String artist, long length)
	{
		this(title, artist, "", length, "");
	}

	/**
	 * Simplified constructor, discarding the album.
	 */
	public Track(String title, String artist, long length, String mbid)
	{
		this(title, artist, "", length, mbid);
	}

	/**
	 * Simplified constructor, discarding the mbid.
	 * 
	 * The length must be specified in seconds.
	 */
	public Track(String title, String artist, String album, long length)
	{
		this(title, artist, album, length, "");
	}

	/**
	 * Constructs a track. The date must be the local time - it will be converted
	 * automatically. The time must be given in seconds since January 1st, 1970, in local time.
	 * It is automatically converted to UTC time.
	 */
	public Track(String title, String artist, String album, long length, String mbid)
	{
		this.title = title;
		this.artist = artist;
		this.album = album;
		this.length = length;
		this.mbid = mbid;
	}
	
	public String getTitle() { return title; }
	
	public String getArtist() { return artist; }
	
	public String getAlbum() { return album; }
	
	/**
	 * Returns the length (in seconds) of the track.
	 */
	public long getLength() { return length; }
	
	public String getMBID() { return mbid; }
	
	public boolean equals(Object obj)
	{
		if (!(obj instanceof Track)) return false;
		Track t = (Track) obj;
		return (getTitle().equals(t.getTitle()) && getArtist().equals(t.getArtist())
				&& getAlbum().equals(t.getAlbum()));
	}
	
	public int hashCode()
	{
		return getTitle().hashCode() + getArtist().hashCode() + 
		getAlbum().hashCode() + getMBID().hashCode();
	}
}
